# Generated by Django 2.1 on 2019-07-11 11:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0088_auto_20190710_0953'),
        ('djconnectwise', '0087_auto_20190709_0836'),
    ]

    operations = [
    ]



# Generated by Django 3.1.2 on 2021-01-08 16:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0141_auto_20201215_1304'),
    ]

    operations = [
        migrations.CreateModel(
            name='CommunicationType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.CharField(max_length=250)),
                ('phone_flag', models.BooleanField(default=False)),
                ('fax_flag', models.BooleanField(default=False)),
                ('email_flag', models.BooleanField(default=False)),
                ('default_flag', models.BooleanField(default=False)),
            ],
        ),
        migrations.CreateModel(
            name='CommunicationTypeTracker',
            fields=[
            ],
            options={
                'db_table': 'djconnectwise_communicationtype',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djconnectwise.communicationtype',),
        ),
    ]



# Generated by Django 2.1.14 on 2019-12-04 08:38

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0110_project_company'),
    ]

    operations = [
        migrations.RenameField(
            model_name='scheduleentry',
            old_name='expected_date_end',
            new_name='date_end',
        ),
        migrations.RenameField(
            model_name='scheduleentry',
            old_name='expected_date_start',
            new_name='date_start',
        ),
    ]



# Generated by Django 3.1.2 on 2020-10-15 14:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0130_activitystatustracker_activitytracker_activitytypetracker_agreementtracker_boardstatustracker_calend'),
    ]

    operations = [
        migrations.AddField(
            model_name='ticket',
            name='udf',
            field=models.JSONField(blank=True, null=True),
        ),
    ]



# Generated by Django 3.1.7 on 2021-06-23 16:27

from django.db import migrations, models
import django.db.models.deletion
import django_extensions.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0150_auto_20210610_1741'),
    ]

    operations = [
        migrations.AddField(
            model_name='item',
            name='inactive_flag',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='subtype',
            name='inactive_flag',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='type',
            name='inactive_flag',
            field=models.BooleanField(default=False),
        ),
        migrations.CreateModel(
            name='TypeSubTypeItemAssociation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('board', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djconnectwise.connectwiseboard')),
                ('item', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='associations', to='djconnectwise.item')),
                ('sub_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='associations', to='djconnectwise.subtype')),
                ('type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='associations', to='djconnectwise.type')),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='TypeSubTypeItemAssociationTracker',
            fields=[
            ],
            options={
                'db_table': 'djconnectwise_type_subtype_item_association',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djconnectwise.typesubtypeitemassociation',),
        ),
    ]



import hashlib
import re
from io import BytesIO

from PIL import Image, ImageOps
from django.conf import settings
from django.core.files.base import ContentFile
from django.core.files.storage import default_storage

_underscorer1 = re.compile(r'(.)([A-Z][a-z]+)')
_underscorer2 = re.compile(r'([a-z0-9])([A-Z])')
FILENAME_EXTENSION_RE = re.compile(r'\.([\w]*)$')


def camel_to_snake(s):
    """
    Is it ironic that this function is written in camel case, yet it
    converts to snake case? hmm..
    """
    subbed = _underscorer1.sub(r'\1_\2', s)
    return _underscorer2.sub(r'\1_\2', subbed).lower()


def snake_to_camel(snake_case_text):
    tokens = snake_case_text.split('_')
    return ''.join(word.capitalize() for word in tokens)


def get_hash(content):
    """Return the hex SHA-1 hash of the given content."""
    return hashlib.sha1(content).hexdigest()


def get_filename_extension(filename):
    """From the given filename, return the extension,
    or None if it can't be parsed.
    """
    m = FILENAME_EXTENSION_RE.search(filename)
    return m.group(1) if m else None


def generate_filename(size, current_filename, extension):
    img_dimensions = 'x'.join([str(i) for i in size])
    filename = '{}{}.{}'.format(current_filename, img_dimensions, extension)
    return filename


def remove_thumbnail(avatar_filename):
    thumbnail_size = {
        'avatar': (80, 80),
        'micro_avatar': (20, 20),
    }
    # This deletes the image name from DB field
    # and also removes thumbnails from storage.
    extension = get_filename_extension(avatar_filename)
    for size in thumbnail_size:
        filename = generate_filename(thumbnail_size[size],
                                     avatar_filename, extension)
        default_storage.delete(filename)

    default_storage.delete(avatar_filename)


def generate_thumbnail(avatar, size, extension, filename):
    filename = generate_filename(size, filename, extension)
    avatar_image = Image.open(BytesIO(avatar))
    thumbnail = ImageOps.fit(avatar_image, size, Image.ANTIALIAS)

    # For now just change the extension to jpeg
    # https://stackoverflow.com/questions/37048807/python-image-library-and-keyerror-jpg
    if extension.lower() == 'jpg':
        extension = 'jpeg'

    byte_stream = BytesIO()
    thumbnail.save(byte_stream, format=extension)
    avatar_file = ContentFile(byte_stream.getvalue())

    return avatar_file, filename


class DjconnectwiseSettings:
    def get_settings(self):
        # Make some defaults
        request_settings = {
            'timeout': 30.0,
            'batch_size': 50,
            'max_attempts': 3,
            'schedule_entry_conditions_size': 0,
            'response_version': '2020.4',
            'sync_child_tickets': True,
            'sync_time_and_note_entries': True,
            'sync_contact_communications': True,
            'keep_closed_ticket_days': 0,
            'keep_closed_status_board_ids': 0
        }

        if hasattr(settings, 'DJCONNECTWISE_CONF_CALLABLE'):
            request_settings.update(settings.DJCONNECTWISE_CONF_CALLABLE())

        return request_settings



# Generated by Django 3.1.1 on 2020-09-30 10:11

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0126_auto_20200924_1320'),
        ('djconnectwise', '0125_auto_20200923_1049'),
    ]

    operations = [
    ]



from django.test import TestCase
from . import mocks
from django.core.files.base import ContentFile
from djconnectwise.utils import get_hash, get_filename_extension, \
                                generate_thumbnail, generate_filename


class TestUtils(TestCase):

    def test_get_hash_returns_hash(self):
        self.assertEqual(
            get_hash('hello, world'.encode('utf-8')),
            'b7e23ec29af22b0b4e41da31e868d57226121c84'
        )

    def test_get_filename_extension_returns_extension(self):
        self.assertEqual(
            get_filename_extension('avatar.jpg'),
            'jpg'
        )
        # It also works with multiple dots in the name
        self.assertEqual(
            get_filename_extension('avatar.one.jpg'),
            'jpg'
        )
        # It also works with spaces in the name
        self.assertEqual(
            get_filename_extension('avatar me.jpg'),
            'jpg'
        )

    def test_get_filename_extension_returns_none_when_invalid(self):
        self.assertEqual(
            get_filename_extension('avatar'),
            None
        )
        self.assertEqual(
            get_filename_extension(''),
            None
        )


class TestThumbnailGeneration(TestCase):
    thumbnail_size = {
        'avatar': (80, 80),
        'micro_avatar': (20, 20),
    }
    filename = 'AnonymousMember.png'
    extension = 'png'
    avatar = mocks.get_member_avatar()

    def test_generate_filename(self):
        expected_filename = 'AnonymousMember.png80x80.png'
        processed_filename = generate_filename(self.thumbnail_size['avatar'],
                                               self.filename, self.extension)

        self.assertEqual(expected_filename, processed_filename)

    def test_generate_avatar_thumbnail(self):
        expected_filename = 'AnonymousMember.png20x20.png'

        file, avatar = generate_thumbnail(self.avatar,
                                          self.thumbnail_size['micro_avatar'],
                                          self.extension, self.filename)

        self.assertEqual(expected_filename, avatar)
        self.assertIsInstance(file, ContentFile)



# Generated by Django 3.1.5 on 2021-01-12 14:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0143_contactcommunication_type'),
    ]

    operations = [
        migrations.AddField(
            model_name='contactcommunication',
            name='default_flag',
            field=models.BooleanField(blank=True, null=True),
        ),
    ]



# Generated by Django 2.1 on 2019-07-29 13:52

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0106_auto_20190722_1524'),
    ]

    operations = [
        migrations.AddField(
            model_name='connectwiseboard',
            name='bill_time',
            field=models.CharField(blank=True, choices=[('Billable', 'Billable'), ('DoNotBill', 'Do Not Bill'), ('NoCharge', 'No Charge')], max_length=50, null=True),
        ),
        migrations.AddField(
            model_name='member',
            name='work_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='djconnectwise.WorkRole'),
        ),
        migrations.AddField(
            model_name='member',
            name='work_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='djconnectwise.WorkType'),
        ),
        migrations.AddField(
            model_name='worktype',
            name='overall_default_flag',
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 3.1.1 on 2020-10-03 11:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0127_merge_20200930_1011'),
    ]

    operations = [
        migrations.AlterField(
            model_name='syncjob',
            name='success',
            field=models.BooleanField(null=True),
        ),
        migrations.AlterField(
            model_name='ticket',
            name='approved',
            field=models.BooleanField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='ticket',
            name='closed_flag',
            field=models.BooleanField(null=True),
        ),
        migrations.AlterField(
            model_name='ticket',
            name='has_child_ticket',
            field=models.BooleanField(null=True),
        ),
        migrations.AlterField(
            model_name='ticket',
            name='is_in_sla',
            field=models.BooleanField(null=True),
        ),
    ]
